# -*- coding: utf-8 -*-

import json
import utils

from flask_wtf import Form
from wtforms import StringField, DateField
from wtforms.ext.sqlalchemy.fields import QuerySelectField
from wtforms.fields.html5 import DateField
from wtforms.validators import Optional, AnyOf
from wtforms.fields.core import UnboundField

from sqlalchemy_utils import escape_like
from sqlalchemy.sql.expression import and_

import shared

from models.inventory import Category, CategoryField, EntryField, EntryStatusType, EntryDocumentationType, EntryUseType, Entry
from models.account import User

import forms.validators

class AdvancedSearch(Form):
	dynamic_fields = {}

	adv_search_name = StringField(
		'adv_search_name',
		id='adv_search_name',
		validators=[Optional(strip_whitespace=True)])

	adv_search_category = QuerySelectField(
		'adv_search_category',
		id='adv_search_category',
		query_factory=lambda: Category.get_flattened(no_category=True),
		get_label='tabbed_name',
		allow_blank=True,
		blank_text=shared.BabelText('select_category'),
		validators=[Optional()])

	adv_search_status = QuerySelectField(
		'adv_search_status',
		id='adv_search_status',
		query_factory=lambda: EntryStatusType.get_flattened(no_status=True),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('select_status'),
		validators=[Optional()])

	adv_search_serial_number = StringField(
		'adv_search_serial_number',
		id='adv_search_serial_number',
		validators=[Optional(strip_whitespace=True)])

	adv_search_in_charge = QuerySelectField(
		'adv_search_in_charge',
		id='adv_search_in_charge',
		query_factory=lambda: User.get_flattened(no_user=shared.BabelText('no_charge')),
		get_label='full_name',
		allow_blank=True,
		blank_text=shared.BabelText('select_charge'),
		validators=[Optional()])

	adv_search_documentation = QuerySelectField(
		'adv_search_documentation',
		query_factory=lambda: EntryDocumentationType.get_flattened(no_documentation=True),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('select_documentation'),
		validators=[Optional()])

	adv_search_description = StringField(
		'adv_search_description',
		id='adv_search_description',
		validators=[Optional(strip_whitespace=True)])

	adv_search_comments = StringField(
		'adv_search_comments',
		id='adv_search_comments',
		validators=[Optional(strip_whitespace=True)])

	adv_search_actual_use_user = QuerySelectField(
		'adv_search_actual_use_user',
		id='adv_search_actual_use_user',
		query_factory=lambda: User.get_flattened(no_user=shared.BabelText('no_use')),
		get_label='full_name',
		allow_blank=True,
		blank_text=shared.BabelText('select_use'),
		validators=[Optional()])

	adv_search_actual_use_type = QuerySelectField(
		'adv_search_actual_use_type',
		id='adv_search_actual_use_type',
		query_factory=lambda: EntryUseType.get_flattened(no_use=True),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('select_use_type'),
		validators=[Optional()])

	@classmethod
	def create_instance(cls):
		def get_field_id(field):
			return field.id if field else None

		def get_field_from_id(type, field):
			if field == 0:
				return type.get_none()
			elif field:
				return type.query.get(field)

			return None

		class DynamicForm(Form):
			dynamic_fields = {}
			category_fields = {}

			def get_query(self):
				fields = [
					self.adv_search_name.data,
					get_field_id(self.adv_search_category.data),
					get_field_id(self.adv_search_status.data),
					self.adv_search_serial_number.data,
					get_field_id(self.adv_search_in_charge.data),
					get_field_id(self.adv_search_documentation.data),
					self.adv_search_description.data,
					self.adv_search_comments.data,
					get_field_id(self.adv_search_actual_use_user.data),
					get_field_id(self.adv_search_actual_use_type.data)]

				extra_fields = {}
				for x in self.dynamic_fields.keys():
					field = getattr(self, x).data
					if field:
						extra_fields[x[len('field_'):]] = field
				fields.append(extra_fields)

				extra_fields = {}
				for x in self.category_fields.keys():
					field = getattr(self, x).data
					if field:
						extra_fields[x[len('category_field_'):]] = field
				fields.append(extra_fields)

				return utils.base64_encode(json.dumps(fields))

			def set_query(self, query):
				fields = json.loads(utils.base64_decode(query))
				self.adv_search_name.data = fields[0]
				self.adv_search_category.data = get_field_from_id(Category, fields[1])
				self.adv_search_status.data = get_field_from_id(EntryStatusType, fields[2])
				self.adv_search_serial_number.data = fields[3]
				self.adv_search_in_charge.data = get_field_from_id(User, fields[4])
				self.adv_search_documentation.data = get_field_from_id(EntryDocumentationType, fields[5])
				self.adv_search_description.data = fields[6]
				self.adv_search_comments.data = fields[7]
				self.adv_search_actual_use_user.data = get_field_from_id(User, fields[8])
				self.adv_search_actual_use_type.data = get_field_from_id(EntryUseType, fields[9])

				for x in fields[10].keys():
					field_name = 'field_' + x
					if hasattr(self, field_name):
						field = getattr(self, field_name)
						field.data = fields[10][x]

				for x in fields[11].keys():
					field_name = 'category_field_' + x
					if hasattr(self, field_name):
						field = getattr(self, field_name)
						field.data = fields[11][x]

			def search(self):
				query = Entry.query
				search = False

				for x in ( # foreign keys
					(self.adv_search_category, Entry.category),
					(self.adv_search_status, Entry.status),
					(self.adv_search_in_charge, Entry.in_charge),
					(self.adv_search_documentation, Entry.documentation),
					(self.adv_search_actual_use_user, Entry.actual_use_user),
					(self.adv_search_actual_use_type, Entry.actual_use_type),):
						if x[0].data != None:
							query = query.filter(x[1] == (x[0].data if x[0].data.id != 0 else None))
							search = True

				for x in ( # text fields
					(self.adv_search_serial_number, Entry.serial_number),):
						if x[0].data:
							query = query.filter(x[1].ilike('%' + escape_like(x[0].data) + '%'))
							search = True
				
				for x in ( # search vectors
					(self.adv_search_name, Entry.name_search_vector),
					(self.adv_search_description, Entry.description_search_vector),
					(self.adv_search_comments, Entry.comments_search_vector),):
						if x[0].data:
							query = query.search(x[0].data, vector=x[1], sort=True)
							search = True

				for x in self.dynamic_fields:
					field = getattr(self, x)
					if field.data:
						if x.startswith('field_'):
							x = x[len('field_'):]
							query = query.filter(and_(
								Entry.fields.has_key(x),
								Entry.fields[x].astext.ilike('%' + escape_like(field.data) + '%')))
							search = True
						elif x.startswith('category_field'):
							x = x[len('category_field_'):]
							query = query.filter(and_(
								Entry.category_fields.has_key(x),
								Entry.category_fields[x].astext.ilike('%' + escape_like(field.data) + '%')))
							search = True

				return query.all() if search else []

		for label, field in vars(cls).items():
			if type(field) is UnboundField and label != 'csrf_token':
				setattr(DynamicForm, label, field)

		for x in EntryField.query.order_by(EntryField.name).all():
			field_name = 'field_' + str(x.id)
			DynamicForm.dynamic_fields[field_name] = x.name
			field = StringField(field_name, id='adv_search_' + field_name, validators=[Optional(strip_whitespace=True)])
			setattr(DynamicForm, field_name, field)

		for x in CategoryField.query.order_by(CategoryField.name).all():
			field_name = 'category_field_' + str(x.id)
			DynamicForm.dynamic_fields[field_name] = x.name
			DynamicForm.category_fields[field_name] = x.category_id
			field = StringField(field_name, id='adv_search_' + field_name, validators=[Optional(strip_whitespace=True)])
			setattr(DynamicForm, field_name, field)

		form = DynamicForm()
		form.csrf_enabled = False
		
		return form