# -*- coding: utf-8 -*-

from datetime import datetime

from shared import db
from flask_sqlalchemy import BaseQuery
from sqlalchemy import func
from sqlalchemy.orm import backref
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy_searchable import SearchQueryMixin, make_searchable
from sqlalchemy.dialects.postgresql import JSONB
from sqlalchemy_utils.types import TSVectorType
from sqlalchemy.sql import exists

import collections
import utils
from flask_babelex import lazy_gettext

from jinja2.utils import escape


class Category(db.Model):
	__tablename__ = 'categories'

	id = db.Column(db.Integer, primary_key=True)
	parent_id = db.Column(db.Integer, db.ForeignKey('categories.id'), index=True, nullable=True)
	name = db.Column(db.String(128), nullable=False)

	parent = db.relationship('Category', backref='children', remote_side='Category.id')

	def __init__(self, name=None, parent=None):
		self.name = name
		self.parent = parent

	@hybrid_property
	def level(self):
		if not self.parent:
			return 0
		else:
			return self.parent.level + 1

	@hybrid_property
	def tabbed_name(self):
		return ('\xa0' * 6 * self.level) + self.name

	@classmethod
	def get_none(cls):
		if not hasattr(cls, 'none'):
			cls.none = cls(lazy_gettext('no_category'), None)
			cls.none.id = 0

		return cls.none

	@classmethod
	def get_parents(cls):
		categories = []

		if db.session.query(Entry.query.filter_by(category_id=None).exists()).scalar():
			categories.append(cls.get_none())

		categories.extend(cls.query
			.filter(cls.parent_id == None)
			.order_by(cls.name)
			.all())

		return categories

	@classmethod
	def get_flattened(cls, no_category=False):
		categories = Category.query.order_by(Category.parent_id.desc(), Category.name).all()
		flattened_categories = []

		if no_category and db.session.query(Entry.query.filter_by(category_id=None).exists()).scalar():
			flattened_categories.append(cls.get_none())

		def get_categories(parent=None, level=0):
			for x in categories:
				if x.parent_id == parent:
					flattened_categories.append(x)
					get_categories(x.id, level + 1)

		get_categories()
		return flattened_categories

	@classmethod
	def get_id_with_pathname(cls, category):
		return str(category.id) + '-' + utils.urlforce(escape(utils.normalize('/'.join(category.pathnames()), allowed_chars='/')))

	def pathnames(self):
		names = collections.deque()
		names.append(self.name)

		parent = self.parent
		while parent is not None:
			names.appendleft(parent.name)
			parent = parent.parent

		return names

	def get_flattened_descendants(self):
		flattened_descendants = []
		def get_children(category=self):
			for x in category.children:
				flattened_descendants.append(x)
				get_children(x)

		get_children()
		return flattened_descendants

db.Index('categories_root_parent_id_name_key', func.lower(Category.name), postgresql_where=Category.parent_id == None, unique=True)
db.Index('categories_parent_id_name_key', Category.parent_id, func.lower(Category.name), postgresql_where=Category.parent_id != None, unique=True)


class CategoryField(db.Model):
	__tablename__ = 'category_fields'

	id = db.Column(db.Integer, primary_key=True)
	category_id = db.Column(db.Integer, db.ForeignKey('categories.id', ondelete='SET NULL'), index=True, nullable=False)
	name = db.Column(db.String(64), nullable=False)

	category = db.relationship('Category', backref='fields', remote_side='Category.id')

db.Index('category_fields_category_id_name_key', 'category_id', func.lower(CategoryField.name), unique=True)


class EntryStatusType(db.Model):
	__tablename__ = 'entry_status_types'

	id = db.Column(db.Integer, primary_key=True)
	name = db.Column(db.String(64), nullable=False)

	def __init__(self, name=None):
		self.name = name

	@classmethod
	def get_none(cls):
		if not hasattr(cls, 'none'):
			cls.none = cls(lazy_gettext('no_status'))
			cls.none.id = 0

		return cls.none

	@classmethod
	def get_flattened(cls, no_status=False):
		status_types = []

		if no_status:
			status_types.append(cls.get_none())

		status_types.extend(cls.query.order_by(cls.name).all())
		return status_types

db.Index('entry_status_types_name_key', func.lower(EntryStatusType.name), unique=True)


class EntryUseType(db.Model):
	__tablename__ = 'entry_use_types'

	id = db.Column(db.Integer, primary_key=True)
	name = db.Column(db.String(64), nullable=False)

	def __init__(self, name=None):
		self.name = name

	@classmethod
	def get_none(cls):
		if not hasattr(cls, 'none'):
			cls.none = cls(lazy_gettext('no_use_type'))
			cls.none.id = 0

		return cls.none

	@classmethod
	def get_flattened(cls, no_use=False):
		use_types = []

		if no_use:
			use_types.append(cls.get_none())

		use_types.extend(cls.query.order_by(cls.name).all())
		return use_types

db.Index('entry_use_types_name_key', func.lower(EntryUseType.name), unique=True)


class EntryDocumentationType(db.Model):
	__tablename__ = 'entry_documentation_types'

	id = db.Column(db.Integer, primary_key=True)
	name = db.Column(db.String(64), nullable=False)

	def __init__(self, name=None):
		self.name = name

	@classmethod
	def get_none(cls):
		if not hasattr(cls, 'none'):
			cls.none = cls(lazy_gettext('no_documentation'))
			cls.none.id = 0

		return cls.none

	@classmethod
	def get_flattened(cls, no_documentation=False):
		documentation_types = []

		if no_documentation:
			documentation_types.append(cls.get_none())

		documentation_types.extend(cls.query.order_by(cls.name).all())
		return documentation_types

db.Index('entry_documentation_types_name_key', func.lower(EntryDocumentationType.name), unique=True)


class EntryField(db.Model):
	__tablename__ = 'entry_fields'

	id = db.Column(db.Integer, primary_key=True)
	name = db.Column(db.String(64), nullable=False)

	def __init__(self, name=None):
		self.name = name

db.Index('entry_fields_name_key', func.lower(EntryField.name), unique=True)


class EntryQuery(BaseQuery, SearchQueryMixin):
	pass


class Entry(db.Model):
	__tablename__ = 'entries'
	query_class = EntryQuery

	id = db.Column(db.Integer, primary_key=True)
	category_id = db.Column(db.Integer, db.ForeignKey('categories.id'), index=True, nullable=True)
	status_id = db.Column(db.Integer, db.ForeignKey('entry_status_types.id', ondelete='SET NULL'), index=True, nullable=True)
	in_charge_id = db.Column(db.Integer, db.ForeignKey('users.id', ondelete='SET NULL'), index=True, nullable=True)
	documentation_id = db.Column(db.Integer, db.ForeignKey('entry_documentation_types.id', ondelete='SET NULL'), index=True, nullable=True)
	actual_use_user_id = db.Column(db.Integer, db.ForeignKey('users.id', ondelete='SET NULL'), index=True, nullable=True)
	actual_use_type_id = db.Column(db.Integer, db.ForeignKey('entry_use_types.id', ondelete='SET NULL'), nullable=True)

	name = db.Column(db.String(256), index=True, nullable=False)
	reception_date = db.Column(db.Date, nullable=False, default=datetime.now())
	serial_number = db.Column(db.String(64), index=True, nullable=True)
	description = db.Column(db.Text, nullable=True)
	comments = db.Column(db.Text, nullable=True)
	fields = db.Column(JSONB(none_as_null=True), nullable=True)
	category_fields = db.Column(JSONB(none_as_null=True), nullable=True)

	actual_use_start_date = db.Column(db.Date, nullable=True)
	actual_use_end_date = db.Column(db.Date, nullable=True)

	image = db.Column(db.String(32), nullable=True)

	name_search_vector = db.Column(TSVectorType('name'))
	description_search_vector = db.Column(TSVectorType('description'))
	comments_search_vector = db.Column(TSVectorType('comments'))
	full_search_vector = db.Column(TSVectorType('name', 'description', 'comments'))

	category = db.relationship('Category', backref='entries')
	status = db.relationship('EntryStatusType')
	in_charge = db.relationship('User', foreign_keys=[in_charge_id])
	documentation = db.relationship('EntryDocumentationType')
	actual_use_type = db.relationship('EntryUseType')
	actual_use_user = db.relationship('User', foreign_keys=[actual_use_user_id])

	@hybrid_property
	def formatted_id(self):
		return self.id + 100000

	def __hash__(self):
		return self.id

	def populate_obj(self, obj):
		obj.name.data = self.name
		obj.category.data = self.category
		obj.status.data = self.status
		obj.reception_date.data = self.reception_date
		obj.serial_number.data = self.serial_number
		obj.in_charge.data = self.in_charge
		obj.documentation.data = self.documentation
		obj.description.data = self.description
		obj.comments.data = self.comments
		obj.actual_use_user.data = self.actual_use_user
		obj.actual_use_type.data = self.actual_use_type
		obj.actual_use_start_date.data = self.actual_use_start_date
		obj.actual_use_end_date.data = self.actual_use_end_date

		if self.fields:
			for x, y in self.fields.items():
				getattr(obj, 'field_' + str(x)).data = y

		if self.category_fields:
			for x, y in self.category_fields.items():
				getattr(obj, 'category_field_' + str(x)).data = y

	def check_use_integrity(self):
		if not self.actual_use_user:
			self.actual_use_type = None
			self.actual_use_start_date = None
			self.actual_use_end_date = None
			

class EntryFile(db.Model):
	__tablename__ = 'entry_files'
	__table_args__ = (db.UniqueConstraint('entry_id', 'name'),)

	id = db.Column(db.Integer, primary_key=True)
	entry_id = db.Column(db.Integer, db.ForeignKey('entries.id'), index=True, nullable=False)

	name = db.Column(db.String(256), nullable=False)
	full_name = db.Column(db.String(1024), nullable=False)
	mime_type = db.Column(db.String(256), nullable=False)
	upload_date = db.Column(db.DateTime(timezone=True), nullable=True, default=datetime.now())

	entry = db.relationship('Entry', backref=backref('files', order_by=full_name), lazy='joined')

	def __init__(self, name=None, full_name=None, mime_type=None):
		self.name = name
		self.full_name = full_name
		self.mime_type = mime_type
