# -*- coding: utf-8 -*-

from flask import Flask, redirect, request, render_template, url_for
from flask_login import LoginManager, AnonymousUserMixin, logout_user, current_user
from flask_wtf.csrf import CsrfProtect
from flask_babelex import Babel, Domain

import functools

import config
import shared
import utils

import models.app
import models.account
import models.inventory

import forms.inventory
import forms.adv_search

from json import JSONEncoder

_default_encoder = JSONEncoder().default
JSONEncoder.default = lambda self, obj: getattr(obj.__class__, "to_json", _default_encoder)(obj)

_app = Flask('photonics')
_app.config.from_object(config.ServerConfig)

_app.jinja_env.filters.update(
	{x.__name__: x
		for x in [getattr(utils, y) for y in dir(utils)]
		if callable(x) and not x.__name__.startswith('__')})

_csrf = CsrfProtect()
_csrf.init_app(_app)

shared.db.init_app(_app)

_login_manager = LoginManager()
_login_manager.init_app(_app)
_login_manager.session_protection = 'strong'

_babel_domain = Domain(config.AppConfig.BABEL_DIRECTORY)
_babel = Babel(default_domain=_babel_domain)
_babel.init_app(_app)

if _app.config['LOGIN_DISABLED']:
	class AnonymousDebugUserMixin(AnonymousUserMixin):
		id = 0
		first_name = u'DEBUG'
		last_name = u'MODE'
		can_login = True
		can_write = True
		can_admin = True
		language = config.AppConfig.DEFAULT_LANGUAGE

	_login_manager.anonymous_user = AnonymousDebugUserMixin
else:
	class AnonymousProductionUserMixin(AnonymousUserMixin):
		id = 0
		first_name = None
		last_name = None
		can_login = False
		can_write = False
		can_Admin = False
		language = config.AppConfig.DEFAULT_LANGUAGE

	_login_manager.anonymous_user = AnonymousProductionUserMixin


@_app.context_processor
def context_processor():
	return {key: value
		for key, value in vars(config.AppConfig).items()
		if not callable(value) and not key.startswith('__')}


@_app.errorhandler(404)
def page_not_found(e):
	return render_template('404.html'), 404


@_app.errorhandler(500)
def page_not_found(e):
	return render_template('500.html'), 500


@_csrf.error_handler
def csrf_error(reason):
	return render_template('400.html', reason=reason), 400


@_login_manager.user_loader
def user_loader(user_id):
	return models.account.User.query.get(user_id)


@_babel.localeselector
def get_locale():
	if current_user.is_authenticated:
		return current_user.language

	return config.AppConfig.DEFAULT_LANGUAGE


def _get_route_path(name):
	name = '/' + name.replace('_', '/')
	return name


def _add_route_query(name, query):
	name += '/' + '/'.join('<' + x + '>' for x in query) if query else ''
	return name


def _create_route(func, route, methods):
	_app.add_url_rule(route, None, func, methods=methods)


def main_route(func):
	_create_route(func, '/', ('GET',))
	return func


def login_route(func):
	_create_route(func, '/login', ('GET', 'POST'))
	_login_manager.login_view = 'login'
	return func


def route(methods=['GET'], query=(), route=None):
	def decorator(func):
		path = _get_route_path(func.__name__) if not route else route
		path = _add_route_query(path, query)
		_create_route(func, path, methods)
		return func
		
	return decorator


def authenticated(anonymous=False, write=None, admin=None):
	def decorator(func):
		@functools.wraps(func)
		def wrapper(*args, **kwargs):
			if anonymous and current_user.is_authenticated:
				return redirect('/')
			elif not anonymous and not _app.config['LOGIN_DISABLED']:
				if not current_user.is_authenticated:
					return redirect(_login_manager.login_view)

				elif not current_user.can_login:
					logout_user()
					return redirect(_login_manager.login_view)

				elif not current_user.can_admin:
					if admin or write and not current_user.can_write:
						return redirect('/')

			return func(*args, **kwargs)

		return functools.update_wrapper(wrapper, func)
	return decorator


def render_anonymous_template(template, **kwargs):
	if 'notification' in kwargs:
		notification = models.app.Notification.query.get(kwargs['notification'])
		if notification:
			kwargs['notification'] = notification.value
		else:
			kwargs['notification'] = None

	return render_template(template, **kwargs)


def render_logged_template(template, **kwargs):
	def add_to_dict(key, value):
		if key not in kwargs:
			kwargs[key] = value()

	for x, y in {
		'adv_search_form': lambda: forms.adv_search.AdvancedSearch.create_instance(),

		'categories': lambda: models.inventory.Category.get_parents(),

		'category_selector': lambda: forms.inventory.CategorySelector(),

		'entry_status_types': lambda:
			models.inventory.EntryStatusType.query
				.order_by(models.inventory.EntryStatusType.name)
				.all(),

		'entry_documentation_types': lambda:
			models.inventory.EntryDocumentationType.query
				.order_by(models.inventory.EntryDocumentationType.name)
				.all(),

		'entry_use_types': lambda:
			models.inventory.EntryUseType.query
				.order_by(models.inventory.EntryUseType.name)
				.all(),

		'entry_fields': lambda:
			models.inventory.EntryField.query
				.order_by(models.inventory.EntryField.name)
				.all(),

		'category_fields': lambda:
			models.inventory.CategoryField.query
				.order_by(models.inventory.CategoryField.name)
				.all(),

		'entry_count': lambda:
			models.inventory.Entry.query.count(),
	}.items():
		add_to_dict(x, y)

	if 'notification' in kwargs:
		notification = models.app.Notification.query.get(kwargs['notification'])
		if notification:
			kwargs['notification'] = notification.value
		else:
			kwargs['notification'] = None
		
	return render_template(template, **kwargs)