# -*- coding: utf-8 -*-

from flask import flash, request, redirect, url_for
from flask_login import LoginManager, login_user, logout_user, current_user
from flask_babelex import gettext

import photon
import config
import utils
from shared import db

import models.account
import forms.account

import random
import string

@photon.login_route
@photon.authenticated(anonymous=True)
def login():
	form = forms.account.Login()

	if request.method == 'POST':
		if not form.validate():
			flash(gettext('invalid_login_form'), 'login')
			return redirect('/login')
		else:
			user = models.account.User.query.filter_by(user_name=form.user_name.data).first()
			if not user or not user.check_password(form.password.data):
				flash(gettext('incorrect_login_data'), 'login')
				return redirect('/login')
			else:
				login_user(user)
				user.update_last_login_date()
				return redirect('/')

	return photon.render_anonymous_template(
		'login.html',
		form=form,
		notification='login_message')


@photon.route()
@photon.authenticated()
def logout():
	logout_user()
	return redirect('/')


@photon.route(methods=('GET', 'POST'), route='/recover_password')
@photon.authenticated(anonymous=True)
def recover_password():
	form = forms.account.RecoverPassword()

	if request.method == 'POST':
		if not form.validate():
			flash(gettext('invalid_recover_password_form'), 'login')
			return redirect('/recover_password')
		else:
			user = models.account.User.query.filter(models.account.User.email == form.email.data).first()
			if user:
				new_password = ''.join(
					random.choice(string.ascii_lowercase + string.ascii_uppercase + string.digits) for _ in range(12))
				user.password = models.account.User.hash_password(new_password)

				try:
					db.session.commit()

					utils.send_mail(
						user.email,
						config.AppConfig.RECOVER_PASSWORD_SUBJECT[user.language],
						config.AppConfig.RECOVER_PASSWORD_CONTENT[user.language]
							.replace('{0}', user.user_name)
							.replace('{1}', new_password))

					flash(gettext('password_recovered_successfully'), 'login_noerror')
				except:
					db.session.rollback()
					flash(gettext('error_recover_password_form'), 'login')
			else:
				flash(gettext('password_recovered_successfully'), 'login_noerror')

			return redirect('/login')
	else:
		return photon.render_anonymous_template(
			'recover_password.html',
			form=form)


@photon.route()
@photon.authenticated()
def profile():
	user = models.account.User.query.get(current_user.id)
	language_form = forms.account.ChangeLanguage()
	password_form = forms.account.ChangePassword()

	language_form.language.data = user.language

	return photon.render_logged_template(
		'profile.html',
		user=user,
		language_form=language_form,
		password_form=password_form)


@photon.route(methods=('POST',), route='/profile/change_language')
@photon.authenticated(write=True)
def profile_change_language():
	form = forms.account.ChangeLanguage()

	if form.validate():
		user = models.account.User.query.get(current_user.id)
		user.language = form.language.data

		try:
			db.session.commit()
		except:
			db.session.rollback()
			flash(gettext('unknown_error'), 'account')
	else:
		errors = utils.reduce_form_errors(form)

		for error in errors:
			flash(error, 'account')

	return redirect(url_for('profile'))


@photon.route(methods=('POST',), route='/profile/change_password')
@photon.authenticated(write=True)
def profile_change_password():
	form = forms.account.ChangePassword()

	if form.validate():
		user = models.account.User.query.get(current_user.id)
		if not user.check_password(form.current_password.data):
			flash(gettext('invalid_profile_current_password_form'), 'account')
		elif form.new_password.data != form.confirm_password.data:
			flash(gettext('invalid_profile_new_password_form'), 'account')
		else:
			user.password = models.account.User.hash_password(form.new_password.data)

			try:
				db.session.commit()
				logout_user()
				flash(gettext('password_changed_successfully'), 'login_noerror')
				return redirect('/')
			except:
				db.session.rollback()
				flash(gettext('unknown_error'), 'account')
	else:
		errors = utils.reduce_form_errors(form)

		for error in errors:
			flash(error, 'account')

	return redirect(url_for('profile'))
