# -*- coding: utf-8 -*-

import sys
import os
import codecs

import functools
import re
import unicodedata

from types import FunctionType

import sqlparse
from sqlparse.sql import Identifier, IdentifierList, Parenthesis
from sqlparse.tokens import Token

from werkzeug.security import generate_password_hash

class Table(object):
    tables = {}

    def __init__(self, name, old_name, columns, **kwargs):
        self.tables[name] = self

        self.name = name
        self.old_name = old_name
        self.rows = []

        self.columns = columns
        self.formatted_columns = ', '.join(map(lambda x: x.name, self.columns))
        for column in self.columns:
            column.table = name

        for k, v in kwargs.items():
            setattr(self, k, v)

    @classmethod
    def get_table_by_old_name(cls, name):
        for table in cls.tables:
            if cls.tables[table].old_name == name:
                return cls.tables[table]

        return None

    def create_new_row(self, data):
        row = {x.name: x.create_new_row_value(data) for x in self.columns}
        for x in self.columns:
            x.check_integrity(row)

        self.rows.append(row)
        return row

    def search_primary_by_column(self, column_name, value):
        for row in self.rows:
            if row[column_name] == value:
                pk = [x for x in self.columns if hasattr(x, 'is_primary') and x.is_primary]
                return row[pk[0].name] if pk else None

        return None

    def format_rows(self):
        formatted_rows = []

        for row in self.rows:
            has_integrity = True

            if (hasattr(self, 'integrity') and self.integrity):
                if type(self.integrity) is list:
                    for t in self.integrity:
                        has_integrity = self.check_integrity(row, t)
                        if not has_integrity:
                            break
                else:
                    has_integrity = self.check_integrity(row, self.integrity)

            if has_integrity:
                formatted_rows.append(
                    u'INSERT INTO {table_name} ({column_names}) VALUES ({row_data});'.format(
                        table_name=self.name,
                        column_names=self.formatted_columns,
                        row_data=self.format_row(row)))

        sequence = self.get_sequence()
        if sequence:
            formatted_rows.append(
                u'ALTER SEQUENCE ' + self.name + '_id_seq RESTART WITH ' + str(sequence + 1) + ';')

        return formatted_rows

    def check_integrity(self, row, integrity):
        if type(integrity) is FunctionType:
            return bool(integrity(row))

        return True

    def get_sequence(self):
        for column in self.columns:
            if hasattr(column, 'is_sequence') and column.is_sequence:
                return column._current_sequence_value

        return 0

    def format_row(self, row):
        return ', '.join([
            self.format_cell(x)
            for x in sorted(
                row.items(),
                key=lambda x: self.find_column_index(x[0]))])

    def format_cell(self, cell):
        value = cell[1]
        column = self.find_column(cell[0])

        if (hasattr(column, 'format') and column.format):
            if type(column.format) is list:
                for t in column.format:
                    value = column.transform_cell(value, t)
            else:
                value = column.transform_cell(value, column.format)

        if value is None:
            return u'NULL'

        t = type(value)

        if t is str:
            return u'\'' + value + u'\''

        if t is bool:
            return str(value).lower()

        return str(value)

    def find_column_index(self, column_name):
        for i, v in enumerate(self.columns):
            if column_name == v.name:
                return i

        return None

    def find_column(self, column_name):
        for c in self.columns:
            if column_name == c.name:
                return c

        return None


class Column(object):
    def __init__(self, name, **kwargs):
        self.name = name

        for k, v in kwargs.items():
            setattr(self, k, v)

        if hasattr(self, 'is_sequence') and self.is_sequence:
            self._current_sequence_value = 0

    def _get_next_sequence(self):
        if not hasattr(self, 'is_sequence') or not self.is_sequence:
            return None

        self._current_sequence_value += 1
        return self._current_sequence_value

    def create_new_row_value(self, data):
        old_table_index = False
        is_sequence = hasattr(self, 'is_sequence') and self.is_sequence

        if hasattr(self, 'old_table_index') and self.old_table_index is not None:
            old_table_index = True
            if type(self.old_table_index) is int and len(data) > self.old_table_index:
                value = data[self.old_table_index]
            elif type(self.old_table_index) is FunctionType:
                value = self.old_table_index(data)
            else:
                value = self.default

            if value is not None and hasattr(self, 'foreign_key') and self.foreign_key:
                getter = value
                if hasattr(self, 'foreign_transform') and self.foreign_transform:
                    if type(self.foreign_transform) is list:
                        getter = value
                        for t in self.foreign_transform:
                            getter = t(getter)
                    else:
                        getter = self.foreign_transform(value)

                if type(self.foreign_key[1]) is FunctionType:
                    get_value = lambda: self.foreign_key[1](getter, Table.tables[self.foreign_key[0]])
                else:
                    get_value = lambda: Table.tables[self.foreign_key[0]].search_primary_by_column(
                        self.foreign_key[1],
                        getter)

                tmp_value = get_value()
                if not tmp_value and hasattr(self, 'create_foreign_key') and self.create_foreign_key:
                    Table.tables[self.foreign_key[0]].create_new_row(self.create_foreign_key(data))
                    value = get_value()
                else:
                    value = tmp_value


        elif is_sequence:
            value = self._get_next_sequence()
        else:
            value = self.default if hasattr(self, 'default') else None

        if hasattr(self, 'transform') and self.transform:
            if type(self.transform) is list:
                for t in self.transform:
                    value = self.transform_cell(value, t)
            else:
                value = self.transform_cell(value, self.transform)

        if old_table_index and is_sequence:
            self._current_sequence_value = value

        return value

    def transform_cell(self, value, transform):
        if type(transform) is FunctionType:
            return transform(value)

        return value

    def check_integrity(self, row):
        if hasattr(self, 'integrity'):
            row[self.name] = self.integrity(row, row[self.name])


def normalize(string):
    string = unicodedata.normalize('NFKD', string)
    string = string.encode('ascii', 'ignore')
    string = string.decode('ascii')
    string = re.sub('[ ]+', ' ', string)
    string = re.sub('[^0-9a-zA-Z ]', '', string)
    return string.lower()

def check_foreign_user(value, table):
    value = normalize(value.strip())

    if value in ['alayn', 'alayn loayssa', 'alayn loayssa lara']:
        value = 'alayn'
    elif value in ['ana']:
        value = 'arodriguez'
    elif value in ['daniel leandro']:
        value = None
    elif value in ['jesus corres']:
        value = None
    elif value in ['manolo lopez amo', 'manueal lopezamo', 'manuel', 'manuel lopez amo', 'manuel lopezamo', 'mnuel lopezamo']:
        value = 'manuellopezamo'
    elif value in ['montse', 'montse fernandez', 'montse fernandez vallejo', 'montse ferni12ndez']:
        value = 'Montse'
    elif value in ['rosana perez']:
        value = 'rosana'
    elif value in ['santiago tainta']:
        value = 'stainta'
    elif value in ['silvia diaz']:
        value = 'sdiaz'
    else:
        value = None

    if not value:
        return None

    return table.search_primary_by_column('user_name', value)


old_schema_tables = [
    'categorias_usuarios',
    'usuarios',
    'categoria',
    'estado',
    'localizacion',
    'uso',
    'elementos']

new_schema_tables = [
    Table('user_types', 'categorias_usuarios', (
        Column('id', is_primary=True, is_sequence=True),
        Column('name', old_table_index=0)),
        integrity=lambda row: row['name'] != 'Otros'),

    Table('users', 'usuarios', (
        Column('id', is_primary=True, is_sequence=True),
        Column(
            'user_type_id',
            old_table_index=lambda x: x[7] if x[6] == 'Otros' else x[6],
            foreign_key=('user_types', 'name'),
            create_foreign_key=lambda x: [x[7]]),
        Column('user_name', old_table_index=0),
        Column('first_name', old_table_index=2, transform=lambda x: x.title()),
        Column('last_name', old_table_index=3, transform=lambda x: x.title()),
        Column('DNI', old_table_index=4, transform=[
            lambda x: x.strip() if x else None,
            lambda x: None if not x else x]),
        Column('email', old_table_index=5),
        Column(
            'password',
            old_table_index=1,
            transform=lambda x: generate_password_hash(
                x,
                method='pbkdf2:sha512:100000',
                salt_length=16)),
        Column(
            'can_login',
            old_table_index=8,
            transform=lambda x: True if x else False),
        Column(
            'can_write',
            old_table_index=8,
            transform=lambda x: True if x in ('Intermedio', 'Administrador') else False),
        Column('can_admin',
            old_table_index=8,
            transform=lambda x: True if x == 'Administrador' else False),
        Column('language', default='es'))),

    Table('categories', 'categoria', (
        Column('id', is_primary=True, is_sequence=True),
        Column('parent_id', default=None),
        Column('name', old_table_index=0, format=[
            lambda x: x.strip().strip('\v\f\t') if x else None,
            lambda x: x.capitalize() if x else None]))),

    Table('entry_status_types', 'estado', (
        Column('id', is_primary=True, is_sequence=True),
        Column('name', old_table_index=0, format=[
            lambda x: 'OK' if x == 'O.K.' else x,
            lambda x: 'KO' if x == 'K.O.' else x,
            lambda x: x.capitalize() if x != 'OK' and x != 'KO' else x]))),

    Table('entry_documentation_types', 'localizacion', (
        Column('id', is_primary=True, is_sequence=True),
        Column('name', old_table_index=0, transform=[
            lambda x: None if x == 'No disponible' else x,
            lambda x: None if x == 'Versión electrónica' else x,
            lambda x: None if x == 'Archivador/Versión electrónica' else x,
            lambda x: None if x == 'Armario manual/v. electrónica' else x])),
        integrity=lambda row: row['name'] != None),

    Table('entry_use_types', 'uso', (
        Column('id', is_primary=True, is_sequence=True),
        Column('name', old_table_index=0, format=[
                lambda x: 'PFC' if x == 'P.F.C.' else x,
                lambda x: x.capitalize() if x != 'PFC' else x,
                lambda x: 'Investigación' if x == 'Investigacion' else x,
                lambda x: 'Prácticas' if x == 'Practicas' else x]))),

    Table('entries', 'elementos', (
        Column(
            'id',
            old_table_index=0,
            is_primary=True,
            is_sequence=True,
            transform=lambda x: int(x) - 100000),
        Column(
            'category_id',
            old_table_index=4,
            foreign_key=('categories', 'name'),
            foreign_transform=lambda x: 'CONTROLADORES DE POLARIZACIÓN' if x == 'CONTROLADORES DE POLARIZACIï¿½N' else x),
        Column('status_id', old_table_index=8, foreign_key=('entry_status_types', 'name')),
        Column(
            'in_charge_id',
            old_table_index=6,
            foreign_key=('users', check_foreign_user)),
        Column(
            'documentation_id',
            old_table_index=7,
            foreign_key=('entry_documentation_types', 'name'),
            foreign_transform=[
                lambda x: 'Archivador' if x == 'Archivador/Versión electrónica' else x,
                lambda x: 'Armario de manuales' if x == 'Armario manual/v. electrónica' else x]),
        Column(
            'actual_use_user_id',
            old_table_index=10,
            foreign_key=('users', check_foreign_user)),
        Column(
            'actual_use_type_id',
            old_table_index=11,
            foreign_key=('entry_use_types', 'name'),
            integrity=lambda row, x: None if not row['actual_use_user_id'] else x),
        Column('name', old_table_index=2, transform=lambda x: x.strip().strip('\v\f\t') if x else None),
        Column('reception_date', old_table_index=1),
        Column(
            'serial_number',
            old_table_index=3,
            transform=[
                lambda x: x.strip().strip('\v\f\t') if x else None,
                lambda x: None if not x or x == 'S/N' else x]),
        Column(
            'description', 
            old_table_index=5,
            transform=[
                lambda x: x.strip().strip('\v\f\t') if x else None,
                lambda x: None if not x else x,
                lambda x: re.sub(' +', ' ', x).strip() if x else None]),
        Column(
            'comments',
            old_table_index=9,
            transform=[
                lambda x: x.strip().strip('\v\f\t') if x else None,
                lambda x: None if not x else x,
                lambda x: re.sub(' +', ' ', x).strip() if x else None]),
        Column('fields', default=None),
        Column('category_fields', default=None),
        Column(
            'actual_use_start_date',
            old_table_index=12,
            transform=lambda x: None if x == '0000-00-00' else x,
            integrity=lambda row, x: None if not row['actual_use_user_id'] else x),
        Column(
            'actual_use_end_date',
            old_table_index=13,
            transform=lambda x: None if x == '0000-00-00' else x,
            integrity=lambda row, x: None if not row['actual_use_user_id'] else x),
        Column('image', default=None)),

        integrity=[
            lambda row: row['name'] is not None and row['name'] != ''])
]


def main(argv):
    if len(argv) == 3 and os.path.isfile(argv[1]) and os.access(argv[1], os.R_OK):
        input_data = parse_old_schema(argv[1])
        mla = ['manuellopezamo', 'manuellopezamo', 'Manuel', 'Lopez-Amo', '', 'mla@unavarra.es', 'Profesor', '', 'Intermedio']
        next(x for x in input_data if x[0] == 'usuarios')[1].append(mla)
        sort_schema(input_data, old_schema_tables)
        output_data = convert_to_new_schema(input_data)
        save_new_schema(argv[2], output_data)
    else:
        print('Usage: ' + sys.argv[0] + ' input_file output_file')


def parse_old_schema(file):
    with open(file, 'r') as f:
        data = f.read()

    data = functools.reduce(
        lambda x, y: x.replace(y, ''),
        ['\0', '\b', '\t', '\v', '\f'],
        data)
    data = sqlparse.parse(data)

    return [parse_old_schema_statement(x)
        for x in data
        if x.get_type().lower() == 'insert']


def parse_old_schema_statement(stmt):
    table = None
    rows = []

    for token in stmt.tokens:
        if isinstance(token, Identifier) and not table:
            table = token.value.strip('`')

        elif isinstance(token, Parenthesis) and table:
            for subtoken in token.tokens:
                if subtoken.ttype is Token.Literal.String.Single:
                    row = [subtoken.value.strip('\'')]
                    break

                elif isinstance(subtoken, IdentifierList):
                    row = [x.value.strip('\'')
                        for x in subtoken.tokens
                        if any(x.ttype is t for t in [
                            Token.Literal.String.Single,
                            Token.Literal.Number.Integer])]
                    break

            rows.append(row)

    return (table, rows)


def sort_schema(data, order):
    for i in sorted(
        (i for i, v in enumerate(data) if v[0] not in order),
        reverse=True):
            data.pop(i)

    ordering = {key: i for i, key in enumerate(order)}
    data.sort(key=lambda x: ordering[x[0]])


def convert_to_new_schema(data):
    return filter(lambda x: x, list(map(convert_table_to_new_schema, data)))


def convert_table_to_new_schema(table):
    new_table = Table.get_table_by_old_name(table[0])
    if not new_table:
        return None

    for row in table[1]:
        new_table.create_new_row(row) 

    return new_table


def save_new_schema(file, data):
    with codecs.open(file, 'w', 'utf-8') as f:
        for table in data:
            for row in table.format_rows():
                print(row, file=f)

            print('', file=f)


if __name__ == '__main__':
    main(sys.argv)