# -*- coding: utf-8 -*-

import sys
import os
import codecs

import hashlib
import io
import lxml.html
import ssl
import urllib.error
import urllib.request

import zipfile


INDEX_URL = 'https://172.18.67.26/FOTONICA/manuales/'
SSL=ssl.SSLContext(ssl.PROTOCOL_SSLv23)


def get_index():
	try:
		data = urllib.request.urlopen(INDEX_URL, context=SSL).read()
	except urllib.error.HTTPError:
		return None

	data = data.decode('iso-8859-1', 'ignore')
	doc = lxml.html.fromstring(data)
	doc.make_links_absolute(INDEX_URL)
	return doc


def get_file_entry_id(name):
	name = name[len('elto_'):]
	ext = name.rfind('.')
	if ext != -1:
		name = name[:ext]

	return int(name) - 100000


def get_file_extension(name):
	index = name .rfind('.')
	return name[index + 1:] if index != -1 else None


def get_file_type(data, ext):
	if ext and ext.lower() == 'txt':
		return 'text/plain'

	if data[:8] == b'\x89PNG\x0D\x0A\x1A\x0A': # PNG
		return 'image/png'

	if data[:3] == b'\xFF\xD8\xFF': # JPG
		return 'image/jpeg'

	if data[:3] == b'\x47\x49\x46': # GIF
		return 'image/gif'

	if data[:4] == b'%PDF': # PDF
		return 'application/pdf'

	if data[:2] == b'PK' and ext.lower() == 'zip': # ZIP
		return 'application/zip'

	if data[:7] == b'Rar!\x1A\x07\x00': # RAR 1.50
		return 'application/x-rar-compressed'

	if data[:8] == b'Rar!\x1A\x07\x01\x00': # RAR 5.0
		return 'application/x-rar-compressed'

	if data[:6] == b'7z\xBC\xAF\x27\x1C': # 7-Zip
		return 'application/x-7z-compressed'

	if data[:2] == b'\x4D\x5A': # EXE
		return 'application/x-msdownload'


def convert_mime_to_extension(mime):
	if mime == 'text/plain':
		return 'txt'

	if mime == 'image/png':
		return 'png'

	if mime == 'image/jpeg':
		return 'jpeg'

	if mime == 'image/gif':
		return 'gif'

	if mime == 'application/pdf':
		return 'pdf'

	if mime == 'application/zip':
		return 'zip'

	if mime == 'application/x-rar-compressed':
		return 'rar'

	if mime == 'application/x-7z-compressed':
		return '7z'

	if mime == 'application/x-msdownload':
		return 'exe'


def fix_name(filename, ext, mime):
	if not ext:
		ext = convert_mime_to_extension(mime)
		if ext:
			filename = filename + '.' + ext

	return filename

def get_hashed_file_name(entry_id, file_name, mime):
	hashed = str(entry_id + 100000) + '_'
	hashed = hashed + hashlib.md5(str(file_name).encode('utf-8')).hexdigest()
	hashed = hashed + '.' + convert_mime_to_extension(mime)
	return hashed


def should_uncompress_file(file):
	num_dirs = 0
	num_nested_dirs = 0
	num_files = 0

	for x in file.infolist():
		if x.file_size == 0:
			slash = '/' if x.filename[-1] == '/' else False
			backslash = '\\' if x.filename[-1] == '\\' else False

			if slash or backslash:
				num_dirs += 1
				if x.filename.index(slash or backslash) + 1 != len(x.filename):
					num_nested_dirs += 1
		else:
			num_files += 1

	# Only uncompress file if no complex directory tree found
	# (it has no folders or only top level folders) and it
	# contains less than 15 files
	if not num_dirs or not num_nested_dirs:
		return num_files <= 15

	return False


def fix_compressed_file(file):
	for x in file.infolist():
		x.filename = x.filename.replace('\\', '/')
		x.orig_filename = x.orig_filename.replace('\\', '/')


def get_compressed_files(entry_id, file):
	files = []
	for x in file.infolist():
		if x.file_size != 0 and x.filename[-1] != '/':
			filename = x.filename
			if filename.rfind('/') != -1:
				filename = filename[filename.rfind('/') + 1:]

			f = file.read(x)
			ext = get_file_extension(filename)
			mime = get_file_type(f, ext)

			if not convert_mime_to_extension(mime):
				return None

			files.append({
				'entry': entry_id,
				'name': get_hashed_file_name(entry_id, x.orig_filename, mime),
				'full_name': fix_name(filename, ext, mime),
				'mime': mime,
				'file': f
			})

	return files


def uncompress_file(entry_id, data, mime):
	file_like = io.BytesIO(data)
	zip = zipfile.ZipFile(file_like)

	if should_uncompress_file(zip):
		fix_compressed_file(zip)
		return get_compressed_files(entry_id, zip)
	
	return None


def parse_urlfile(f):
	try:
		data = urllib.request.urlopen(f['url'], context=SSL).read()
	except urllib.error.HTTPError:
		return []

	entry_id = get_file_entry_id(f['name'])
	ext = get_file_extension(f['name'])
	mime_type = get_file_type(data, ext)

	if mime_type == 'application/zip':
		uncompressed_files = uncompress_file(entry_id, data, mime_type)
		if uncompressed_files:
			internal_files = uncompressed_files
			return internal_files

	internal_files = [{
		'entry': entry_id,
		'name': get_hashed_file_name(entry_id, entry_id, mime_type),
		'full_name': fix_name(f['name'], ext, mime_type),
		'mime': mime_type,
		'file': data
	}]

	return internal_files


def format_row(f):
	return ','.join([
		str(f['entry']),
		'\'' + f['name'] + '\'',
		'\'' + f['full_name'].replace('\\\\', '\\').replace('\'', '\\''') + '\'',
		'\'' + f['mime'] + '\'',
		'NULL'
	])


def create_query(f):
	formatted_row = format_row(f)
	query = 'INSERT INTO entry_files (entry_id, name, full_name, mime_type, upload_date) VALUES ({row});'

	return query.format(row=formatted_row)


def save_file(path, file):
	with open(os.path.join(path, file['name']), 'wb') as f:
		f.write(file['file'])



def save_queries(file, queries):
	with codecs.open(file, 'w', 'utf-8') as f:
		for query in queries:
			print(query, file=f)


if __name__ == '__main__':
	if len(sys.argv) == 3 and os.path.isdir(sys.argv[1]) and os.access(sys.argv[1], os.W_OK):
		doc = get_index()
		anchors = doc.xpath('//table//td/a')

		files = []
		queries = []

		for x in anchors:
			href = x.attrib.get('href')
			if INDEX_URL + x.text == x.attrib.get('href'):
				files.append({'name': x.text, 'url': href})

		for x in files:
			entry_files = parse_urlfile(x)
			for y in entry_files:
				queries.append(create_query(y))
				save_file(sys.argv[1], y)

		save_queries(sys.argv[2], queries)
