# -*- coding: utf-8 -*-

from flask_wtf import Form
from wtforms import HiddenField, BooleanField, IntegerField, StringField, PasswordField, TextAreaField, FieldList, SelectField, FileField
from wtforms.ext.sqlalchemy.fields import QuerySelectField
from wtforms.fields.html5 import EmailField
from wtforms.widgets import HiddenInput
from wtforms.validators import DataRequired, Optional, EqualTo, Regexp

import shared
from config import AppConfig

import models.account

import forms.validators

class SimpleItemAdd(Form):
	name = StringField('name', validators=[
		DataRequired(message=shared.BabelText('simple_item_add_name_required')),
		forms.validators.CustomLength(max=64, message=shared.BabelText('simple_item_length'))])


class SimpleItemEdit(Form):
	id = IntegerField('id', widget=HiddenInput(), validators=[DataRequired()])
	name = StringField('name', validators=[
		DataRequired(message=shared.BabelText('simple_item_edit_name_required')),
		forms.validators.CustomLength(max=64, message=shared.BabelText('simple_item_length'))])


class SimpleItemDelete(Form):
	id = IntegerField('id', widget=HiddenInput(), validators=[DataRequired()])


class CategoryAdd(Form):
	parent = IntegerField('parent', widget=HiddenInput(), validators=[Optional(strip_whitespace=True)])
	name = StringField(
		'name',
		validators=[
			DataRequired(message=shared.BabelText('category_name')),
			forms.validators.CustomLength(max=128, message=shared.BabelText('category_length'))])


class CategoryEdit(Form):
	id = IntegerField('id', widget=HiddenInput(), validators=[DataRequired()])
	name = StringField('name', validators=[
		DataRequired(message=shared.BabelText('category_edit_name_required')),
		forms.validators.CustomLength(max=128, message=shared.BabelText('category_length'))])

	fields = FieldList(StringField('field', validators=[
		DataRequired(message=shared.BabelText('category_field_edit_name_required')),
		forms.validators.CustomLength(max=64, message=shared.BabelText('category_field_length'))]))

	new_fields = FieldList(StringField('new_field', validators=[
		DataRequired(message=shared.BabelText('category_field_add_name_required')),
		forms.validators.CustomLength(max=64, message=shared.BabelText('category_field_length'))]))
	

class Notifications(Form):
	login_message = TextAreaField('login_message', validators=[Optional(strip_whitespace=True)])
	home_message = TextAreaField('home_message', validators=[Optional(strip_whitespace=True)])
	new_entry_message = TextAreaField('new_entry_message', validators=[Optional(strip_whitespace=True)])


class UserAdd(Form):
	user_name = StringField('user_name', validators=[
		DataRequired(message=shared.BabelText('user_user_name_required')),
		forms.validators.CustomLength(max=64, message=shared.BabelText('admin_username_length')),
		Regexp('^[a-zA-Z0-9]+$', message=shared.BabelText('admin_username_alphanumeric'))])

	first_name = StringField(
		'first_name',
		validators=[
			DataRequired(message=shared.BabelText('user_first_name_required')),
			forms.validators.CustomLength(max=128)])

	last_name = StringField(
		'last_name',
		validators=[
			DataRequired(message=shared.BabelText('user_last_name_required')),
			forms.validators.CustomLength(max=128)])

	dni = StringField(
		'dni',
		validators=[
			Optional(strip_whitespace=True),
			forms.validators.CustomLength(max=16, message=shared.BabelText('user_dni_length'))])

	email = EmailField(
		'email',
		validators=[
			DataRequired(message=shared.BabelText('user_email_required')),
			forms.validators.CustomLength(max=256, message=shared.BabelText('user_email_length'))])

	user_type = QuerySelectField(
		'user_type',
		query_factory=lambda: models.account.UserType.query.order_by(models.account.UserType.name).all(),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('none'),
		validators=[Optional()])
	
	password = PasswordField(
		'password',
		validators=[
			DataRequired(message=shared.BabelText('user_password_required')),
			forms.validators.CustomLength(min=6, message=shared.BabelText('user_password_length'))])

	confirm_password = PasswordField(
		'confirm_password',
		validators=[
			DataRequired(message=shared.BabelText('user_password_required')),
			forms.validators.CustomLength(min=6),
			EqualTo('password', message=shared.BabelText('user_password_match'))])

	can_login = BooleanField('can_login')
	can_write = BooleanField('can_write')
	can_admin = BooleanField('can_admin')

	language = SelectField(
		'language',
		choices=AppConfig.AVAILABLE_LANGUAGES,
		default=AppConfig.DEFAULT_LANGUAGE)


class UserEdit(Form):
	user_name = StringField('user_name', validators=[
		DataRequired(),
		forms.validators.CustomLength(max=64, message=shared.BabelText('admin_username_length')),
		Regexp('^[a-zA-Z0-9]+$', message=shared.BabelText('admin_username_alphanumeric'))])

	first_name = StringField(
		'first_name',
		validators=[
			DataRequired(message=shared.BabelText('user_first_name_required')),
			forms.validators.CustomLength(max=128, message=shared.BabelText('user_first_name_length'))])

	last_name = StringField(
		'last_name',
		validators=[
			DataRequired(message=shared.BabelText('user_last_name_required')),
			forms.validators.CustomLength(max=128, message=shared.BabelText('user_last_name_length'))])

	dni = StringField(
		'dni',
		validators=[
			Optional(strip_whitespace=True),
			forms.validators.CustomLength(max=16, message=shared.BabelText('user_dni_length'))])

	email = EmailField(
		'email',
		validators=[
			DataRequired(message=shared.BabelText('user_email_required')),
			forms.validators.CustomLength(max=256, message=shared.BabelText('user_email_length'))])

	user_type = QuerySelectField(
		'user_type',
		query_factory=lambda: models.account.UserType.query.order_by(models.account.UserType.name).all(),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('none'),
		validators=[Optional()])
	
	password = PasswordField(
		'password',
		validators=[
			Optional(),
			forms.validators.CustomLength(min=6, message=shared.BabelText('user_password_length'))])
	
	confirm_password = PasswordField(
		'confirm_password',
		validators=[
			Optional(),
			forms.validators.CustomLength(min=6),
			EqualTo('password', message=shared.BabelText('user_password_match'))])

	can_login = BooleanField('can_login')
	can_write = BooleanField('can_write')
	can_admin = BooleanField('can_admin')

	language = SelectField('language', choices=AppConfig.AVAILABLE_LANGUAGES)

	def populate_obj(self, obj):
		obj.user_name = self.user_name.data
		obj.first_name = self.first_name.data
		obj.last_name = self.last_name.data
		obj.dni = self.dni.data
		obj.email = self.email.data
		obj.user_type = self.user_type.data
		obj.can_login = self.can_login.data
		obj.can_write = self.can_write.data
		obj.can_admin = self.can_admin.data
		obj.language = self.language.data


class BackupRestore(Form):
	backup_file = FileField('backup_file')
