# -*- coding: utf-8 -*-

import datetime

from flask_wtf import Form
from wtforms import IntegerField, StringField, TextAreaField, FileField, HiddenField, FieldList
from wtforms.ext.sqlalchemy.fields import QuerySelectField
from wtforms.validators import DataRequired, Optional, AnyOf
from wtforms.widgets import HiddenInput
from wtforms.fields.core import UnboundField

import shared

from models.inventory import Category, CategoryField, EntryField, EntryStatusType, EntryDocumentationType, EntryUseType
from models.account import User

import forms.fields
import forms.validators

class CategorySelector(Form):
	category = QuerySelectField(
		'category',
		id='category_selector',
		query_factory=Category.get_flattened,
		get_pk=Category.get_id_with_pathname,
		get_label='tabbed_name',
		allow_blank=True,
		blank_text=shared.BabelText('select_category'))


class EntryDelete(Form):
	id = IntegerField('id', widget=HiddenInput(), validators=[DataRequired()])


class Entry(Form):
	dynamic_fields = {}

	name = StringField('name', validators=[
			DataRequired(message=shared.BabelText('inventory_name_required')),
			forms.validators.CustomLength(max=256, message=shared.BabelText('inventory_name_length'))])

	category = QuerySelectField(
		'category',
		query_factory=Category.get_flattened,
		get_label='tabbed_name',
		allow_blank=True,
		blank_text=shared.BabelText('select_category'),
		validators=[DataRequired(message=shared.BabelText('inventory_category_required'))])

	status = QuerySelectField(
		'status',
		query_factory=lambda: EntryStatusType.query.order_by(EntryStatusType.name).all(),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('no_status'),
		validators=[Optional()])

	reception_date = forms.fields.DateField(
		'reception_date',
		format='%Y-%m-%d',
		message=shared.BabelText('date_format'),
		default=datetime.date.today(),
		validators=[DataRequired(message=shared.BabelText('inventory_reception_date_required'))])

	serial_number = StringField(
		'serial_number',
		validators=[
			Optional(strip_whitespace=True),
			forms.validators.CustomLength(max=64, message=shared.BabelText('inventory_serial_number_length'))])

	in_charge = QuerySelectField(
		'in_charge',
		query_factory=lambda: User.query.order_by(User.first_name, User.last_name).all(),
		get_label='full_name',
		allow_blank=True,
		blank_text=shared.BabelText('no_charge'),
		validators=[Optional()])

	documentation = QuerySelectField(
		'documentation',
		query_factory=lambda: EntryDocumentationType.query.order_by(EntryDocumentationType.name).all(),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('no_documentation'),
		validators=[Optional()])

	description = TextAreaField(
		'description',
		validators=[Optional(strip_whitespace=True)])

	comments = TextAreaField(
		'comments',
		validators=[Optional(strip_whitespace=True)])

	actual_use_user = QuerySelectField(
		'actual_use_user',
		query_factory=lambda: User.query.order_by(User.first_name, User.last_name).all(),
		get_label='full_name',
		allow_blank=True,
		blank_text=shared.BabelText('no_use'),
		validators=[Optional()])

	actual_use_type = QuerySelectField(
		'actual_use_type',
		query_factory=lambda: EntryUseType.query.order_by(EntryUseType.name).all(),
		get_label='name',
		allow_blank=True,
		blank_text=shared.BabelText('no_use_type'),
		validators=[Optional()])

	actual_use_start_date = forms.fields.DateField(
		'actual_use_start_date',
		format='%Y-%m-%d',
		message=shared.BabelText('date_format'),
		validators=[Optional()])

	actual_use_end_date = forms.fields.DateField(
		'actual_use_end_date',
		format='%Y-%m-%d',
		message=shared.BabelText('date_format'),
		validators=[Optional()])

	image = FileField('image')
	delete_image = HiddenField('delete_image')

	files = FileField('files')
	delete_file = FieldList(StringField('delete_file'))

	@classmethod
	def create_instance(cls):
		class DynamicForm(Form):
			dynamic_fields = {}
			category_fields = {}

			def populate_obj(self, obj):
				obj.name = self.name.data
				obj.category = self.category.data
				obj.status = self.status.data
				obj.reception_date = self.reception_date.data
				obj.serial_number = self.serial_number.data
				obj.in_charge = self.in_charge.data
				obj.documentation = self.documentation.data
				obj.description = self.description.data
				obj.comments = self.comments.data
				obj.actual_use_user = self.actual_use_user.data
				obj.actual_use_type = self.actual_use_type.data
				obj.actual_use_start_date = self.actual_use_start_date.data
				obj.actual_use_end_date = self.actual_use_end_date.data

				fields = {}
				for x in self.dynamic_fields.keys():
					if x.find('field_') == 0:
						attr = getattr(self, x).data
						if attr:
							fields[x[len('field_'):]] = attr

				obj.fields = fields if len(fields.keys()) else None

				fields = {}
				category_fields = self.category.data.fields

				for x in self.dynamic_fields.keys():
					if x.find('category_field_') == 0:
						field_id = x[len('category_field_'):]
						if next((x.id for x in category_fields if x.id == int(field_id)), None):
							attr = getattr(self, x).data
							if attr:
								fields[field_id] = attr

				obj.category_fields = fields if len(fields.keys()) else None

		for label, field in vars(cls).items():
			if type(field) is UnboundField and label != 'csrf_token':
				setattr(DynamicForm, label, field)

		setattr(DynamicForm, 'xhr', HiddenField('xhr'))

		for x in EntryField.query.order_by(EntryField.name).all():
			field_name = 'field_' + str(x.id)
			DynamicForm.dynamic_fields[field_name] = x.name
			field = TextAreaField(field_name, validators=[Optional(strip_whitespace=True)])
			setattr(DynamicForm, field_name, field)

		for x in CategoryField.query.order_by(CategoryField.name).all():
			field_name = 'category_field_' + str(x.id)
			DynamicForm.dynamic_fields[field_name] = x.name
			DynamicForm.category_fields[field_name] = x.category_id
			field = TextAreaField(field_name, validators=[Optional(strip_whitespace=True)])
			setattr(DynamicForm, field_name, field)

		return DynamicForm()
