# -*- coding: utf-8 -*-

import shared

from werkzeug.security import generate_password_hash, check_password_hash
from flask_login import UserMixin
from shared import db
from sqlalchemy.ext.hybrid import hybrid_property

from datetime import datetime
from flask_babelex import lazy_gettext
from config import AppConfig

class UserType(db.Model):
	__tablename__ = 'user_types'

	id = db.Column(db.Integer, primary_key=True)
	name = db.Column(db.String(64), index=True, unique=True, nullable=False)

	def __init__(self, name=None):
		self.name = name


class User(db.Model, UserMixin):
	__tablename__ = 'users'

	id = db.Column(db.Integer, primary_key=True)
	user_type_id = db.Column(db.Integer, db.ForeignKey('user_types.id', ondelete='SET NULL'), index=True, nullable=True)
	user_name = db.Column(db.String(64), index=True, unique=True, nullable=False)
	first_name = db.Column(db.String(128), nullable=False)
	last_name = db.Column(db.String(128), nullable=False)
	dni = db.Column(db.String(16), nullable=True)
	email = db.Column(db.String(256), index=True, unique=True, nullable=False)
	password = db.Column(db.String(256), nullable=False)
	can_login = db.Column(db.Boolean, nullable=False)
	can_write = db.Column(db.Boolean, nullable=False)
	can_admin = db.Column(db.Boolean, nullable=False)
	language = db.Column(db.String(4), default=AppConfig.DEFAULT_LANGUAGE, nullable=False)

	user_type = db.relationship('UserType')

	@hybrid_property
	def full_name(self):
		return ((str(self.first_name) if self.first_name else '') + ' ' + (self.last_name if self.last_name else '')).strip()

	@property
	def is_active(self):
		return self.can_login

	@classmethod
	def create_instance(cls, obj):
		user = cls()

		user.user_name = obj.user_name.data
		user.first_name = obj.first_name.data
		user.last_name = obj.last_name.data
		user.dni = obj.dni.data
		user.email = obj.email.data
		user.password = generate_password_hash(obj.password.data)
		user.can_login = obj.can_login.data
		user.can_write = obj.can_write.data
		user.can_admin = obj.can_admin.data
		
		user.language = obj.language.data
		user.user_type = obj.user_type.data

		return user

	@classmethod
	def hash_password(cls, password):
		return generate_password_hash(
			password,
			method='pbkdf2:sha512:100000',
			salt_length=16)

	@classmethod
	def get_none(cls, no_user='no_user'):
		if not hasattr(cls, 'none'):
			cls.none = {}

		str = no_user
		if type(str) is shared.BabelText:
			str = repr(str)

		if str not in cls.none:
			cls.none[str] = User()
			cls.none[str].id = 0
			cls.none[str].first_name = no_user

		return cls.none[str]

	@classmethod
	def get_flattened(cls, no_user=False):
		users = []

		if no_user:
			users.append(cls.get_none(no_user))

		users.extend(cls.query.order_by(cls.first_name, cls.last_name).all())
		return users

	def populate_obj(self, obj):
		obj.user_name.data = self.user_name
		obj.first_name.data = self.first_name
		obj.last_name.data = self.last_name
		obj.email.data = self.email
		obj.password.data = generate_password_hash

	def get_id(self):
		return str(self.id)

	def update_last_login_date(self):
		self.last_login_date = datetime.now()
		db.session.commit()

	def check_password(self, password):
		return check_password_hash(self.password, password)