# -*- coding: utf-8 -*-

from flask import flash, request, redirect, url_for, send_from_directory, abort
from flask_babelex import gettext
from jinja2.utils import escape

import photon
import utils
from config import AppConfig
from shared import db

from models.inventory import Category, CategoryField, Entry, EntryField, EntryFile
import forms.inventory

from sqlalchemy import cast, String
from sqlalchemy_utils import escape_like
from sqlalchemy.exc import IntegrityError

import collections
import os

from orderedset import OrderedSet

@photon.main_route
@photon.route()
@photon.authenticated()
def inventory():
	return photon.render_logged_template(
		'categories.html',
		notification='home_message',
		entries=None)


@photon.route(query=('int:entry_id', 'path:entry_name'))
@photon.authenticated()
def inventory_entry(entry_id, entry_name):
	entry = Entry.query.get(entry_id)
	if not entry or utils.urlforce(escape(utils.normalize(entry.name))) != entry_name:
		abort(404)
		return

	delete_form = forms.inventory.EntryDelete()
	delete_form.id.data = entry.id

	return photon.render_logged_template(
		'inventory.html',
		main_title=gettext('inventory'),
		titles=None,
		notification='home_message',
		collapsable_entries=False,
		entries=[entry] if entry else [],
		delete_form=delete_form)


@photon.route(query=('int:category_id','path:category_name'))
@photon.authenticated()
def inventory_category(category_id, category_name):
	if category_id == 0:
		category = Category.get_none()
	else:
		category = Category.query.get(category_id)
	
		if not category or utils.urlforce(escape(utils.normalize('/'.join(category.pathnames()), allowed_chars='/'))) != category_name:
			abort(404)
			return

	titles = collections.deque()
	titles.appendleft(category.name)

	parent_category = category
	while parent_category.parent:
		parent_category = parent_category.parent
		titles.appendleft(parent_category.name)

	return photon.render_logged_template(
		'inventory.html',
		main_title=gettext('category'),
		titles=titles,
		notification='home_message',
		collapsable_entries=True,
		entries=Entry.query
			.filter_by(category_id=None if category_id == 0 else category_id)
			.order_by(Entry.id.desc())
			.all(),
		category_id=category.id,
		delete_form=None)


@photon.route()
@photon.authenticated()
def inventory_all():
	return photon.render_logged_template(
		'inventory.html',
		main_title=gettext('inventory'),
		titles=None,
		notification='home_message',
		collapsable_entries=True,
		entries=Entry.query
			.order_by(Entry.id.desc())
			.all(),
		delete_form=None)


@photon.route(query=('int:entry_id',))
@photon.authenticated()
def inventory_image(entry_id):
	entry = Entry.query.get(entry_id)
	if not entry or not entry.image:
		abort(404)

	return send_from_directory(
		AppConfig.IMAGES_DIRECTORY,
		entry.image,
		add_etags=False,
		cache_timeout=0)


@photon.route(query=('int:id', 'string:filename'))
@photon.authenticated()
def inventory_file(id, filename):
	entry_file = EntryFile.query.get(id)
	if not entry_file or entry_file.full_name != filename:
		abort(404)

	return send_from_directory(
		AppConfig.FILES_DIRECTORY,
		entry_file.name,
		add_etags=False,
		cache_timeout=0)


@photon.route(query=('path:search_query',))
@photon.authenticated()
def inventory_search(search_query):
	if not search_query.startswith('query/'):
		return redirect('/')

	search_query = search_query[len('query/'):]
	search_query = search_query.strip()
	entries = OrderedSet()

	if search_query.isdigit():
		entries.update(Entry.query
			.filter(cast(Entry.formatted_id, String).like('%' + search_query + '%'))
			.order_by(Entry.formatted_id)
			.all())

	if len(search_query) >= 3 and len(search_query.split()) == 1:
		entries.update(Entry.query
			.filter(Entry.serial_number.ilike('%' + escape_like(search_query) + '%'))
			.all())

	entries.update(Entry.query.search(search_query, vector=Entry.name_search_vector, sort=True).all())
	entries.update(Entry.query.search(search_query, vector=Entry.description_search_vector, sort=True).all())
	entries.update(Entry.query.search(search_query, vector=Entry.comments_search_vector, sort=True).all())
	entries.update(Entry.query.search(search_query, vector=Entry.full_search_vector, sort=True).all())

	return photon.render_logged_template(
		'inventory.html',
		main_title=gettext('search'),
		notification='home_message',
		titles=[search_query],
		collapsable_entries=True,
		entries=entries,
		delete_form=None)


@photon.route(methods=('POST',), route='/inventory/advanced_search')
@photon.authenticated()
def inventory_advanced_search():
	form = forms.adv_search.AdvancedSearch.create_instance()
	return redirect(
		url_for('inventory_advanced_search_query',
		search_query=form.get_query()))


@photon.route(methods=('GET',), route='/inventory/advanced_search/<path:search_query>')
@photon.authenticated()
def inventory_advanced_search_query(search_query):
	form = forms.adv_search.AdvancedSearch.create_instance()
	form.set_query(search_query)

	if form.validate():
		entries = form.search()
	else:
		entries = []

		errors = utils.reduce_form_errors(form)

		if not errors:
			flash(gettext('invalid_advanced_search_form'), 'inventory')

		for error in errors:
			flash(error, 'inventory')

	return photon.render_logged_template(
		'inventory.html',
		main_title=gettext('advanced_search'),
		notification='home_message',
		collapsable_entries=True,
		entries=entries,
		adv_search_form=form,
		delete_form=None)


@photon.route(methods=('GET', 'POST'))
@photon.authenticated(write=True)
def inventory_add():
	form = forms.inventory.Entry.create_instance()

	if request.method == 'POST':
		if form.validate():
			entry = Entry()
			form.populate_obj(entry)
			entry.check_use_integrity()
			db.session.add(entry)

			if form.xhr.data == 'xhr' and form.image.data:
				image_filename = str(entry.id) + utils.get_file_extension(form.image.data.filename)
				entry.image = image_filename
				
			try:
				db.session.commit()

				if form.xhr.data == 'xhr':
					if form.image.data:
						form.image.data.save(os.path.join(AppConfig.IMAGES_DIRECTORY, image_filename))

					if form.files.raw_data:
						for f in form.files.raw_data:
							filename = str(entry.id) + '_' + utils.get_md5_uuid() + utils.get_file_extension(f.filename)
							f.save(os.path.join(AppConfig.FILES_DIRECTORY, filename))
							new_file = EntryFile(filename, f.filename, f.mimetype)
							new_file.entry = entry
							db.session.add(new_file)
						db.session.commit()
					
				return_url = url_for(
					'inventory_entry',
					entry_id=entry.id,
					entry_name=utils.urlforce(escape(utils.normalize(entry.name))))

				if form.xhr.data == 'xhr':
					return return_url
				else:
					return redirect(return_url)

			except:
				db.session.rollback()
				flash(gettext('unknown_error'), 'inventory')

		else:
			errors = utils.reduce_form_errors(form)

			if not errors:
				flash(gettext('invalid_add_entry_form'), 'inventory')

			for error in errors:
				flash(error, 'admin')

	if request.method == 'POST' and form.xhr.data == 'xhr':
		return '__Error'

	return photon.render_logged_template(
		'inventory_form.html',
		title=gettext('new_entry'),
		notification='new_entry_message',
		form=form)


@photon.route(methods=('GET', 'POST'), query=('int:entry_id',))
@photon.authenticated(write=True)
def inventory_edit(entry_id):
	form = forms.inventory.Entry.create_instance()
	entry = Entry.query.get(entry_id)

	if not entry:
		abort(404)

	if request.method == 'POST':
		if form.validate():
			form.populate_obj(entry)
			entry.check_use_integrity()

			if (entry.image and form.delete_image.data == 'true') or (form.xhr.data == 'xhr' and form.image.data):
				try:
					os.remove(os.path.join(AppConfig.IMAGES_DIRECTORY, entry.image))
				except:
					pass
					
				entry.image = None

			for x in form.delete_file:
				f = EntryFile.query.get(int(x.name[len('delete_file-'):]))
				if f and f.entry_id == entry.id:
					try:
						os.remove(os.path.join(AppConfig.FILES_DIRECTORY, f.name))
					except:
						pass
					db.session.delete(f)

			if form.xhr.data == 'xhr' and form.image.data:
				image_filename = str(entry.id) + utils.get_file_extension(form.image.data.filename)
				entry.image = image_filename

			try:
				db.session.commit()

				if form.xhr.data == 'xhr':
					if form.image.data:
						form.image.data.save(os.path.join(AppConfig.IMAGES_DIRECTORY, image_filename))

					if form.files.raw_data:
						for f in form.files.raw_data:
							filename = str(entry.id) + '_' + utils.get_md5_uuid() + utils.get_file_extension(f.filename)
							f.save(os.path.join(AppConfig.FILES_DIRECTORY, filename))
							new_file = EntryFile(filename, f.filename, f.mimetype)
							new_file.entry = entry
							db.session.add(new_file)
						db.session.commit()

				return_url = url_for(
					'inventory_entry',
					entry_id=entry.id,
					entry_name=utils.urlforce(escape(utils.normalize(entry.name))))

				if form.xhr.data == 'xhr':
					return return_url
				else:
					return redirect(return_url)

			except:
				db.session.rollback()
				flash(gettext('unknown_error'), 'inventory')
			
		else:
			errors = utils.reduce_form_errors(form)
			if not errors:
				flash(gettext('invalid_edit_entry_form'), 'inventory')

			for error in errors:
				flash(error, 'inventory')

	else:
		entry.populate_obj(form)

	if request.method == 'POST' and form.xhr.data == 'xhr':
		return '__Error'

	return photon.render_logged_template(
		'inventory_form.html',
		title=gettext('edit_entry'),
		notification='new_entry_message',
		id=entry.id,
		form=form,
		files=entry.files,
		image=entry.image)


@photon.route(methods=('POST',))
@photon.authenticated(write=True)
def inventory_delete():
	form = forms.inventory.EntryDelete()

	if form.validate():
		entry = Entry.query.get(form.id.data)
		if not entry:
			return redirect('/')

		category = entry.category

		if entry.image:
			try:
				os.remove(os.path.join(AppConfig.IMAGES_DIRECTORY, entry.image))
			except:
				pass

		if entry.files:
			for f in entry.files:
				try:
					os.remove(os.path.join(AppConfig.FILES_DIRECTORY, f.name))
				except:
					pass
					
				db.session.delete(f)

		db.session.delete(entry)
		db.session.commit()

		if category:
			return redirect(url_for(
				'inventory_category',
				category_id=category.id,
				category_name=utils.urlforce(escape(utils.normalize('/'.join(category.pathnames()), allowed_chars='/')))))

	return redirect('/')
